changeset 50401:465c72f54419

(asm-calculate-indentation, asm-indent-line): New functions. (asm-mode): Use them in indent-line-function. Set tab-always-indent to nil to stay closer to previous tab behavior. Set comment-add. Remove space at end of comment-add. (asm-newline): Obsolete. Use newline-and-indent instead. (asm-mode-map): Remove TAB binding. (asm-code-level-empty-comment-pattern) (asm-flush-left-empty-comment-pattern) (asm-inline-empty-comment-pattern): Remove. (asm-comment): Use newcomment functions instead. (asm-colon): Don't tab if colon is not after a label. (asm-line-matches, asm-pop-comment-level): Remove.
author Stefan Monnier <monnier@iro.umontreal.ca>
date Wed, 02 Apr 2003 16:44:32 +0000
parents 95fdaebcdfcb
children f6e3a528d815
files lisp/progmodes/asm-mode.el
diffstat 1 files changed, 75 insertions(+), 68 deletions(-) [+]
line wrap: on
line diff
--- a/lisp/progmodes/asm-mode.el	Wed Apr 02 10:41:40 2003 +0000
+++ b/lisp/progmodes/asm-mode.el	Wed Apr 02 16:44:32 2003 +0000
@@ -75,9 +75,8 @@
     ;; Note that the comment character isn't set up until asm-mode is called.
     (define-key map ":"		'asm-colon)
     (define-key map "\C-c;"	'comment-region)
-    (define-key map "\C-i"	'tab-to-tab-stop)
-    (define-key map "\C-j"	'asm-newline)
-    (define-key map "\C-m"	'asm-newline)
+    (define-key map "\C-j"	'newline-and-indent)
+    (define-key map "\C-m"	'newline-and-indent)
     map)
   "Keymap for Asm mode.")
 
@@ -88,10 +87,6 @@
     2 font-lock-keyword-face))
  "Additional expressions to highlight in Assembler mode.")
 
-(defconst asm-code-level-empty-comment-pattern "^[\t ]+\\s<\\s< *$")
-(defconst asm-flush-left-empty-comment-pattern "^\\s<\\s<\\s< *$")
-(defconst asm-inline-empty-comment-pattern "^.+\\s<+ *$")
-
 ;;;###autoload
 (defun asm-mode ()
   "Major mode for editing typical assembler code.
@@ -119,6 +114,9 @@
   (setq local-abbrev-table asm-mode-abbrev-table)
   (make-local-variable 'font-lock-defaults)
   (setq font-lock-defaults '(asm-font-lock-keywords))
+  (set (make-local-variable 'indent-line-function) 'asm-indent-line)
+  ;; Stay closer to the old TAB behavior (was tab-to-tab-stop).
+  (set (make-local-variable 'tab-always-indent) nil)
 
   (run-hooks 'asm-mode-set-comment-hook)
   ;; Make our own local child of asm-mode-map
@@ -129,53 +127,59 @@
   (modify-syntax-entry	asm-comment-char "<")
 
   (make-local-variable 'comment-start)
-  (setq comment-start (string asm-comment-char ?\ ))
+  (setq comment-start (string asm-comment-char))
+  (make-local-variable 'comment-add)
+  (setq comment-add 1)
   (make-local-variable 'comment-start-skip)
   (setq comment-start-skip "\\(?:\\s<+\\|/\\*+\\)[ \t]*")
   (make-local-variable 'comment-end-skip)
-  (setq comment-end-skip  "[ \t]*\\(\\s>\\|\\*+/\\)")
+  (setq comment-end-skip "[ \t]*\\(\\s>\\|\\*+/\\)")
   (make-local-variable 'comment-end)
   (setq comment-end "")
   (setq fill-prefix "\t")
-  (run-hooks 'asm-mode-hook))
-
+  (run-mode-hooks 'asm-mode-hook))
+
+(defun asm-indent-line ()
+  "Auto-indent the current line."
+  (interactive)
+  (let* ((savep (point))
+	 (indent (condition-case nil
+		     (save-excursion
+		       (forward-line 0)
+		       (skip-chars-forward " \t")
+		       (if (>= (point) savep) (setq savep nil))
+		       (max (asm-calculate-indentation) 0))
+		   (error 0))))
+    (if savep
+	(save-excursion (indent-line-to indent))
+      (indent-line-to indent))))
+
+(defun asm-calculate-indentation ()
+  (or
+   ;; Flush labels to the left margin.
+   (and (looking-at "\\(\\sw\\|\\s_\\)+:") 0)
+   ;; Same thing for `;;;' comments.
+   (and (looking-at "\\s<\\s<\\s<") 0)
+   ;; Simple `;' comments go to the comment-column.
+   (and (looking-at "\\s<\\(\\S<\\|\\'\\)") comment-column)
+   ;; The rest goes at the first tab stop.
+   (or (car tab-stop-list) tab-width)))
+
 (defun asm-colon ()
   "Insert a colon; if it follows a label, delete the label's indentation."
   (interactive)
-  (save-excursion
-    (beginning-of-line)
-    (if (looking-at "[ \t]+\\(\\sw\\|\\s_\\)+$")
-	(delete-horizontal-space)))
-  (insert ":")
-  (tab-to-tab-stop)
-  )
-
-(defun asm-newline ()
-  "Insert LFD + fill-prefix, to bring us back to code-indent level."
-  (interactive)
-  (if (eolp) (delete-horizontal-space))
-  (insert "\n")
-  (tab-to-tab-stop)
-  )
+  (let ((labelp nil))
+    (save-excursion
+      (skip-syntax-backward "w_")
+      (skip-syntax-backward " ")
+      (if (setq labelp (bolp)) (delete-horizontal-space)))
+    (call-interactively 'self-insert-command)
+    (when labelp
+      (delete-horizontal-space)
+      (tab-to-tab-stop))))
 
-(defun asm-line-matches (pattern)
-  (save-excursion
-    (beginning-of-line)
-    (looking-at pattern)))
-
-(defun asm-pop-comment-level ()
-  ;; Delete an empty comment ending current line.  Then set up for a new one,
-  ;; on the current line if it was all comment, otherwise above it
-  (end-of-line)
-  (delete-horizontal-space)
-  (while (= (preceding-char) asm-comment-char)
-    (delete-backward-char 1))
-  (delete-horizontal-space)
-  (if (bolp)
-      nil
-    (beginning-of-line)
-    (open-line 1)))
-
+;; Obsolete since Emacs-21.4.
+(defalias 'asm-newline 'newline-and-indent)
 
 (defun asm-comment ()
   "Convert an empty comment to a `larger' kind, or start a new one.
@@ -188,40 +192,43 @@
 Suggested usage:  while writing your code, trigger asm-comment
 repeatedly until you are satisfied with the kind of comment."
   (interactive)
+  (comment-normalize-vars)
+  (let (comempty comment)
+    (save-excursion
+      (beginning-of-line)
+      (setq comment (comment-search-forward (line-end-position) t))
+      (setq comempty (looking-at "[ \t]*$")))
+
   (cond
 
    ;; Blank line?  Then start comment at code indent level.
-   ((asm-line-matches "^[ \t]*$")
-    (delete-horizontal-space)
-    (tab-to-tab-stop)
-    (insert asm-comment-char comment-start))
+   ;; Just like `comment-dwim'.  -stef
+   ((save-excursion (beginning-of-line) (looking-at "^[ \t]*$"))
+    (indent-according-to-mode)
+    (insert asm-comment-char asm-comment-char ?\ ))
 
-   ;; Nonblank line with no comment chars in it?
-   ;; Then start a comment at the current comment column
-   ((asm-line-matches (format "^[^%c\n]+$" asm-comment-char))
+   ;; Nonblank line w/o comment => start a comment at comment-column.
+   ;; Also: point before the comment => jump inside.
+   ((or (null comment) (< (point) comment))
     (indent-for-comment))
 
-   ;; Flush-left comment present?  Just insert character.
-   ((asm-line-matches asm-flush-left-empty-comment-pattern)
+   ;; Flush-left or non-empty comment present => just insert character.
+   ((or (not comempty) (save-excursion (goto-char comment) (bolp)))
     (insert asm-comment-char))
 
-   ;; Empty code-level comment already present?
-   ;; Then start flush-left comment, on line above if this one is nonempty.
-   ((asm-line-matches asm-code-level-empty-comment-pattern)
-    (asm-pop-comment-level)
-    (insert asm-comment-char asm-comment-char comment-start))
+   ;; Empty code-level comment => upgrade to next comment level.
+   ((save-excursion (goto-char comment) (skip-chars-backward " \t") (bolp))
+    (goto-char comment)
+    (insert asm-comment-char)
+    (indent-for-comment))
 
-   ;; Empty comment ends line?
-   ;; Then make code-level comment, on line above if this one is nonempty.
-   ((asm-line-matches asm-inline-empty-comment-pattern)
-    (asm-pop-comment-level)
-    (tab-to-tab-stop)
-    (insert asm-comment-char comment-start))
-
-   ;; If all else fails, insert character
+   ;; Empty comment ends non-empty code line => new comment above.
    (t
-    (insert asm-comment-char)))
-  (end-of-line))
+    (goto-char comment)
+    (skip-chars-backward " \t")
+    (delete-region (point) (line-end-position))
+    (beginning-of-line) (insert "\n") (backward-char)
+    (asm-comment)))))
 
 (provide 'asm-mode)